/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.tools.edit.behaviour;

import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.command.UndoableComposite;
import net.refractions.udig.project.command.UndoableMapCommand;
import net.refractions.udig.tools.edit.Behaviour;
import net.refractions.udig.tools.edit.EditPlugin;
import net.refractions.udig.tools.edit.EditState;
import net.refractions.udig.tools.edit.EditToolHandler;
import net.refractions.udig.tools.edit.commands.AddFeaturesCommand;
import net.refractions.udig.tools.edit.commands.SplitFeatureCommand;
import net.refractions.udig.tools.edit.support.GeometryCreationUtil;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.geotools.filter.Filter;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.LineString;

/**
 * <p>Requirements:
 * <ul>
 * <li>Current Shape is not null</li>
 * <li>At least one different shape on black board</li>
 * </ul>
 * </p>
 * <p>Action:
 * <ul>
 * <li>Splits all the geoms on the blackboard (except the current geom) using 
 * the current Shape</li>
 * </ul>
 * </p>
 * @author jones
 * @since 1.1.0
 */
public class SplitGeometryBehaviour implements Behaviour {



    public boolean isValid( EditToolHandler handler ) {
        return handler.getCurrentShape()!=null;
    }

    public UndoableMapCommand getCommand( EditToolHandler handler ) {
        List<UndoableMapCommand> commands = new ArrayList<UndoableMapCommand>();
		LineString line = GeometryCreationUtil.createGeom(LineString.class,
				handler.getCurrentShape());
		ILayer editLayer = handler.getEditLayer();
		Envelope bounds = line.getEnvelopeInternal();
		if (bounds.getWidth() < 0.000001)
			bounds.init(bounds.getMinX() - 0.5, bounds.getMaxX() + 0.5, bounds
					.getMinY(), bounds.getMaxY());
		if (bounds.getHeight() < 0.000001)
			bounds.init(bounds.getMinX(), bounds.getMaxX(),
					bounds.getMinY() - 0.5, bounds.getMaxY() + 0.5);

		Filter filter = editLayer.createBBoxFilter(bounds,
				new NullProgressMonitor());
		commands.add(new AddFeaturesCommand(handler
				.getEditBlackboard(editLayer), editLayer, filter));

		commands.add(new SplitFeatureCommand(handler, EditState.NONE));
        return new UndoableComposite(commands);
    }

    public void handleError( EditToolHandler handler, Throwable error, UndoableMapCommand command ) {
        EditPlugin.log("", error); //$NON-NLS-1$
    }

}
