/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.tools.edit.commands;

import net.refractions.udig.project.command.AbstractCommand;
import net.refractions.udig.project.command.UndoableMapCommand;
import net.refractions.udig.tool.edit.internal.Messages;
import net.refractions.udig.tools.edit.support.EditBlackboard;
import net.refractions.udig.tools.edit.support.Point;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Moves Vertex in the {@link net.refractions.udig.tools.edit.support.EditBlackboard}
 * @author jones
 * @since 1.1.0
 */
public class MoveVertexCommand extends AbstractCommand implements UndoableMapCommand{

    EditBlackboard bb;
    Point src;
    private Point dest;
    private boolean ran=false;
    private boolean redoUndoOnly=false;
    
    /**
     * 
     * @param bb blackboard to manipulate
     * @param src point to move
     * @param dest where to move point.
     * @param coords the coords to move from the src point location.  If null all points will be moved.
     */
    public MoveVertexCommand( EditBlackboard bb, Point src, Point dest){
        this.bb=bb;
        this.src=src;
        this.dest=dest;
    }
    
    public void run( IProgressMonitor monitor ) throws Exception {
        if( !ran && redoUndoOnly ){
            // this is a bit of a hack, see setRedoUndoOnly.  
            ran=true;
            return;
        }

         bb.moveCoords(src.getX(), src.getY(), dest.getX(), dest.getY() );
    }

    public String getName() {
        return Messages.MoveVertexCommand_name;
    }

    public void rollback( IProgressMonitor monitor ) throws Exception {
        bb.moveCoords(dest.getX(), dest.getY(), src.getX(), src.getY());
    }
    
    /**
     * This determines whether the first time the method is ran whether it will do anything.
     * <p>
     * In the case of the MoveVertex behaviour the vertices are moved
     * little by little, pixel by pixel often.  So undoing and redoing all that would be 
     * pointless.  For those cases this command has a RedoUndoOnly parameter so that when the command
     * is sent to the map it does nothing.  But once undo is called redo will still work.  If this is false then 
     * it will operate normally.
     * </p>
     * @param redoUndoOnly
     */
    public void setRedoUndoOnly(boolean redoUndoOnly ){
        this.redoUndoOnly=redoUndoOnly;
    }

}
