/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.tools.edit.commands;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.geom.Ellipse2D;
import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.command.AbstractCommand;
import net.refractions.udig.project.command.UndoableComposite;
import net.refractions.udig.project.command.UndoableMapCommand;
import net.refractions.udig.project.internal.ProjectPlugin;
import net.refractions.udig.project.ui.AnimationUpdater;
import net.refractions.udig.project.ui.IAnimation;
import net.refractions.udig.project.ui.commands.AbstractDrawCommand;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.tool.IToolContext;
import net.refractions.udig.tool.edit.internal.Messages;
import net.refractions.udig.tools.edit.EditPlugin;
import net.refractions.udig.tools.edit.EditState;
import net.refractions.udig.tools.edit.EditToolHandler;
import net.refractions.udig.tools.edit.support.EditBlackboard;
import net.refractions.udig.tools.edit.support.EditGeom;
import net.refractions.udig.tools.edit.support.Point;
import net.refractions.udig.tools.edit.support.ShapeType;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.FeatureSource;
import org.geotools.data.FeatureStore;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.geotools.filter.BBoxExpression;
import org.geotools.filter.Filter;
import org.geotools.filter.FilterFactory;
import org.geotools.filter.FilterFactoryFinder;
import org.geotools.filter.FilterType;
import org.geotools.filter.GeometryFilter;
import org.geotools.geometry.jts.JTS;
import org.geotools.geometry.jts.ReferencedEnvelope;
import org.opengis.referencing.operation.MathTransform;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;

/**
 * Searches a layer for an intersection between a mouse click and a feature and selects the features
 * if found.
 * 
 * @author Jesse
 * @since 1.1.0
 */
public class SelectGeometryCommand extends AbstractCommand implements UndoableMapCommand {

    private EditToolHandler handler;
    private Class[] acceptableClasses;
    private boolean onlyAdd;
    private boolean permitClear;
    private MapMouseEvent event;
    private short filterType;

    private UndoableMapCommand command;
    private ShapeType toCreate;

    public SelectGeometryCommand( EditToolHandler handler, MapMouseEvent e,
            Class[] acceptableClasses, short filterType, boolean permitClear, boolean onlyAdd ) {
        this.handler = handler;
        this.event = e;
        Class[] c=new Class[0];
        if( acceptableClasses!=null ){
            c=new Class[acceptableClasses.length];
            System.arraycopy(acceptableClasses, 0, c, 0, c.length);
        }
        this.acceptableClasses = c;

        this.filterType = filterType;
        this.onlyAdd = onlyAdd;
        this.permitClear = permitClear;
    }

    public void run( IProgressMonitor monitor ) throws Exception {
        if (command == null) {
            IToolContext context = handler.getContext();
            ILayer editLayer = handler.getEditLayer();
                        
            EditBlackboard editBlackboard = handler.getEditBlackboard(editLayer);
            editBlackboard.startBatchingEvents();
            BlockingSelectionAnim animation = new BlockingSelectionAnim(event.x, event.y);
            try {
                AnimationUpdater.runTimer(context.getMapDisplay(),animation);
                try {
                    FeatureStore store = editLayer.getResource(FeatureStore.class, null);
                    ReferencedEnvelope bbox = handler.getContext().getBoundingBox(event.getPoint(),
                            7);
                    Filter createBBoxFilter = createBBoxFilter(bbox, editLayer, filterType);
                    FeatureCollection collection = store.getFeatures(createBBoxFilter);
                    FeatureIterator reader = collection.features();
                    try {
                        boolean hasFeature = false;
                        try {
                            hasFeature = reader.hasNext();
                        } catch (Exception e) {
                            reader.close();
                            createBBoxFilter = createBBoxFilter(bbox, editLayer, FilterType.GEOMETRY_BBOX);
                            collection = store.getFeatures(createBBoxFilter);
                            reader = collection.features();
                            try{
                            hasFeature = reader.hasNext();
                            }catch (Exception e2) {
                                EditPlugin.log("",e2); //$NON-NLS-1$
                            }
                        }
                        if (hasFeature) {
                            selectAllIntersectedFeatures(monitor, editLayer, reader);
                        } else {
                            if (!event.isModifierDown(MapMouseEvent.MOD1_DOWN_MASK) && !event.isShiftDown() && permitClear) {
                                writeModifiedFeaturesAndStartEditing(monitor, editBlackboard);
                            }
                        }

                        if (command == null && toCreate != null) {
                            List<UndoableMapCommand> commands = new ArrayList<UndoableMapCommand>();
                            commands.add(handler.getContext().getEditFactory()
                                    .createNullEditFeatureCommand());
                            commands.add(new StartEditingCommand(handler, this.event, toCreate));
                        }
                    } finally {
                        collection.close(reader);
                    }
                } catch (Exception e2) {
                    EditPlugin.log("", e2); //$NON-NLS-1$
                }
            } finally {
                if (animation != null) {
                    animation.setValid(false);
                    animation = null;
                }
                editBlackboard.fireBatchedEvents();
            }
        } else {
            command.run(monitor);
        }
    }

    /**
     *
     * @param monitor
     * @param editBlackboard
     * @throws Exception
     */
    private void writeModifiedFeaturesAndStartEditing( IProgressMonitor monitor, EditBlackboard editBlackboard ) throws Exception {
        List<UndoableMapCommand> commands = new ArrayList<UndoableMapCommand>();
        if (hasDirtyGeom())
            commands.add(handler.getCommand(handler.getAcceptBehaviours()));
        commands.add(new RemoveEditGeomCommand(handler, editBlackboard
                .getGeoms()));
        if (toCreate != null) {
            commands.add(handler.getContext().getEditFactory()
                    .createNullEditFeatureCommand());
            if( toCreate==ShapeType.POINT ){
                commands.add(new StartEditingCommand(handler, this.event,
                    toCreate, handler.getCurrentState()));
                commands.add(handler.getCommand(handler
                        .getAcceptBehaviours()));
            }else{
                commands.add(new StartEditingCommand(handler, this.event,
                        toCreate, EditState.CREATING));
            }
        }
        this.command = new UndoableComposite(commands);
        setAndRun(monitor, command);
    }

    /**
     *
     * @param monitor
     * @param selectedLayer
     * @param reader
     * @throws Exception
     */
    private void selectAllIntersectedFeatures( IProgressMonitor monitor, ILayer selectedLayer, FeatureIterator reader ) throws Exception {
        List<UndoableMapCommand> commands = new ArrayList<UndoableMapCommand>();
        boolean firstPass = true;
        GeometryFactory factory=new GeometryFactory();
        Envelope temp = handler.getContext().getPixelBoundingBox(event.getPoint());
        temp=JTS.transform(temp, selectedLayer.mapToLayerTransform() );
        Geometry pixelBoundingBox = factory.toGeometry(temp);
        List<Feature> allFeatures=new ArrayList<Feature>();
        Feature intersectedFeature=processReader(reader, allFeatures, pixelBoundingBox);
        
       for( Feature feature : allFeatures ) {
            if( intersectedFeature!=null && !feature.getDefaultGeometry().intersects(pixelBoundingBox) )
                continue;
            for( Class<Geometry> clazz : acceptableClasses ) {
                if (onlyAdd || handler.getCurrentGeom() != null
                        && (event.isShiftDown() || event.isModifierDown(MapMouseEvent.MOD1_DOWN_MASK))) {
                    setAndRun(monitor, new AddGeomCommand(handler, feature, Point.valueOf(event.x, event.y)));
                } else {
                    if (clazz.isAssignableFrom(feature.getDefaultGeometry().getClass())) {

                        if (firstPass) {
                            commands.add(handler.getCommand(handler
                                    .getAcceptBehaviours()));
                            commands
                                .add(new SetGeomCommand(handler, feature,
                                        selectedLayer, Point.valueOf(event.x,
                                                event.y)));
                            firstPass = false;
                        } else {
                            commands.add(new AddGeomCommand(handler, feature, null));
                        }
                        EditPlugin
                                .trace(
                                        EditPlugin.SELECTION,
                                        "Feature is one of the acceptable classes " + feature.getID(), null); //$NON-NLS-1$
                    } else {
                        EditPlugin
                                .trace(
                                        EditPlugin.SELECTION,
                                        "Feature is not one of the acceptable classes " + feature.getID(), null); //$NON-NLS-1$
                    }
                }
            }
        }
        UndoableComposite undoableComposite = new UndoableComposite(
                commands);
        undoableComposite.setName(Messages.SelectGeometryCommand_name);
        setAndRun(monitor, undoableComposite);
    }

    private Feature processReader( FeatureIterator reader, List<Feature> allFeatures, Geometry pixelBoundingBox ) {

        Feature intersectingFeature=null;
        
        while ( reader.hasNext() ){
            Feature feature=reader.next();
            if( intersectingFeature==null && feature.getDefaultGeometry().intersects(pixelBoundingBox )){
                intersectingFeature=feature;
            }
            allFeatures.add(feature);
        }
        return intersectingFeature;
    }

    private boolean hasDirtyGeom() {
        if (handler.getCurrentGeom() != null && handler.getCurrentGeom().isChanged())
            return true;
        List<EditGeom> geoms = handler.getEditBlackboard(handler.getEditLayer())
                .getGeoms();
        for( EditGeom geom : geoms ) {
            if (geom.isChanged())
                return true;
        }
        return false;
    }

    /**
     * Creates A geometry filter for the given layer.
     * 
     * @param boundingBox in the same crs as the viewport model.
     * @return a Geometry filter in the correct CRS or null if an exception occurs.
     */
    public Filter createBBoxFilter( Envelope boundingBox, ILayer layer, short filterType ) {
        FilterFactory factory = FilterFactoryFinder.createFilterFactory();
        GeometryFilter bboxFilter = null;
        if (!layer.hasResource(FeatureSource.class))
            return Filter.ALL;
        try {

            Envelope bbox;
            try {
                MathTransform transform = layer.mapToLayerTransform();
                bbox = JTS.transform(boundingBox, transform);
            } catch (Exception e) {
                bbox = boundingBox;
            }
            BBoxExpression bb = factory.createBBoxExpression(bbox);
            bboxFilter = factory.createGeometryFilter(filterType);
            bboxFilter.addRightGeometry(bb);

            String geom = layer.getSchema().getDefaultGeometry().getName();

            bboxFilter.addLeftGeometry(factory.createAttributeExpression(geom));
        } catch (Exception e) {
            ProjectPlugin.getPlugin().log(e);
        }
        return bboxFilter;
    }

    private void setAndRun( IProgressMonitor monitor, UndoableMapCommand undoableComposite )
            throws Exception {
        this.command = undoableComposite;
        command.setMap(getMap());
        command.run(monitor);
    }

    public String getName() {
        return Messages.SelectGeometryCommand_name;
    }

    public void rollback( IProgressMonitor monitor ) throws Exception {
        command.rollback(monitor);
    }

    private static class BlockingSelectionAnim extends AbstractDrawCommand implements IAnimation {
        private int x;
        private int y;

        BlockingSelectionAnim( int x, int y ) {
            this.x = x;
            this.y = y;
        }

        int frame = 2;

        public short getFrameInterval() {
            return 250;
        }

        public void nextFrame() {
            frame -= 2;
            if (frame < 0)
                frame = 6;
        }

        public boolean hasNext() {
            return isValid();
        }

        public void run( IProgressMonitor monitor ) throws Exception {
            graphics.setColor(Color.RED);
            int rad = 3 + frame;
            graphics.draw(new Ellipse2D.Float(x - rad, y - rad, rad * 2, rad * 2));
        }

        public Rectangle getValidArea() {
            int rad = 3 + frame;
            return new Rectangle(x - rad, y - rad, rad * 2, rad * 2);
        }

    }

    public void setCreateOnNoSelect( ShapeType toCreate ) {
        this.toCreate = toCreate;
    }

}
