package net.refractions.udig.tools.edit.commands;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.command.AbstractCommand;
import net.refractions.udig.project.command.UndoableMapCommand;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.tool.IToolContext;
import net.refractions.udig.tools.edit.EditToolHandler;
import net.refractions.udig.tools.edit.support.PrimitiveShape;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.geotools.geometry.jts.ReferencedEnvelope;

/**
 * Sets the feature currently being edited in the edit manager. 
 * @author chorner
 * @since 1.1.0
 */
public class SetEditFeatureCommand extends AbstractCommand implements UndoableMapCommand {

    EditToolHandler handler;
    MapMouseEvent event;
    PrimitiveShape shape;
    Feature oldFeature;
    ILayer oldLayer;
    
    public SetEditFeatureCommand (EditToolHandler handler, MapMouseEvent e, PrimitiveShape shape) {
        this.handler = handler;
        this.event = e;
        this.shape = shape;
    }
    
    public void run( IProgressMonitor monitor ) throws Exception {
        IToolContext context = handler.getContext();
        ReferencedEnvelope bbox = handler.getContext().getBoundingBox(event.getPoint(),7);
        FeatureCollection fc = context.getFeaturesInBbox(handler.getEditLayer(), bbox);
        FeatureIterator it = fc.features();
        Feature feature = null;
        while (it.hasNext()) {
            Feature feat = it.next();
            if (feat.getID().equals(shape.getEditGeom().getFeatureIDRef().toString())) {
                feature = feat;
                break;
            }
        }
        it.close();
        oldFeature = getMap().getEditManagerInternal().getEditFeature();
        oldLayer = getMap().getEditManagerInternal().getEditLayer();
        getMap().getEditManagerInternal().setEditFeature(feature, (Layer) handler.getEditLayer());
    }

    public String getName() {
        return null;
    }

    public void rollback( IProgressMonitor monitor ) throws Exception {
        getMap().getEditManagerInternal().setEditFeature(oldFeature, (Layer) oldLayer);
    }

}
